#pragma once

/****************************************************************************************
* Author:	Gilles Bellot
* Date:		27/06/2018 - Lenningen - Luxembourg
*
* Desc:		geometry
*
* History: - 20/03/2019: computation of the centroid of convex polygons
*
* ToDo:
****************************************************************************************/

// INCLUDES /////////////////////////////////////////////////////////////////////////////

// Direct2D
#include <d2d1_3.h>

// C++
#include <vector>


// CLASSES //////////////////////////////////////////////////////////////////////////////

namespace util
{
	template<typename T>
	class Expected;
}

namespace mathematics
{
	
	class Geometry
	{
	private:

	public:
		Geometry() {};
		~Geometry() {};

		// helper functions
		static void computeCoordinatesOnEllipse(const D2D1_ELLIPSE& ellipse, float angle, D2D1_POINT_2F& point);	// computes the x and y-coordinates of a point on an ellipse given by the angle

		// distance
		static float distance2D(const D2D1_POINT_2F& A, const D2D1_POINT_2F& B);									// computes the distance between two points in euclidean space
		static float squareDistance2D(const D2D1_POINT_2F& A, const D2D1_POINT_2F& B);								// computes the square distance between two points in the euclidean plane

		// polygons
		static void computeCentroid(const std::vector<D2D1_POINT_2F>& vertices, D2D1_POINT_2F* centroid);			// this function computes the centroid of a convex and closed polygon
		static void computeBoundingBox(const std::vector<D2D1_POINT_2F>& vertices, D2D1_POINT_2F* leftTop, D2D1_POINT_2F* rightBottom);	// computes the bounding box for a polygon
	};
}
