#pragma once

/****************************************************************************************
* Author:	Gilles Bellot
* Date:		15/03/2019 - Lenningen - Luxembourg
*
* Desc:		This class models kinematics using various types of integration
*
* History:	- 27/03/2019: added a symplectic integrator for one-dimensional kinematics
			- 29/03/2019: added a symplectic integrator for two-dimensional motion
*
* ToDo:		- lots
****************************************************************************************/

// INCLUDES /////////////////////////////////////////////////////////////////////////////

// CLASSES //////////////////////////////////////////////////////////////////////////////

namespace util
{
	template<typename T>
	class Expected;
}

namespace mathematics
{
	struct Vector2F;
}

namespace physics
{
	class Kinematics
	{
	private:
		
	public:
		// unit conversions
		static const float metersPerPixel;														// 1 pixel = 0.0002645 meters
		static const float pixelsPerMeter;														// 1 meter = 3779.5275590551 pixels

		// one-dimensional kinematics
		static float posUM(const float x0, const float v0, const float a, double dt);			// computes the position after a time t (assume uniform acceleration)
		static void semiImplicitEuler(float& pos, float& vel, const float acc, const double dt);	// updates position, velocity and acceleration of an object using semi-implicit Euler (symplectic)

		// two-dimensional kinematics
		static void semiImplicitEuler(mathematics::Vector2F& pos, mathematics::Vector2F& vel, const mathematics::Vector2F acc, const double dt);
	};
}
