#include "vectors.h"

#include <math.h>

namespace mathematics
{
	/////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// Lengths //////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////
	
	// square length
	float Vector2F::getSquareLength()
	{
		// if the length was already compute, return it
		squareLength = x * x + y * y;
		return squareLength;
	}

	// length
	float Vector2F::getLength()
	{
		length = sqrtf(x * x + y * y);
		return length;
	}

	/////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// Normalize ////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////
	void Vector2F::normalize(float l)
	{
		if (l == -1)
		{
			// no length provided -> compute it
			l = getLength();
		}

		// normalize vector
		x = x / l;
		y = y / l;
	}

	/////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// Operators ////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////
	Vector2F& Vector2F::operator*=(const float a)
	{
		x *= a;
		y *= a;

		return *this;
	}

	Vector2F& Vector2F::operator*(const float a)
	{
		Vector2F temp;
		temp.x = x * a;
		temp.y = y * a;

		return *this;
	}

	Vector2F& Vector2F::operator*=(const double a)
	{
		x *= (float)a;
		y *= (float)a;

		return *this;
	}

	Vector2F& Vector2F::operator*(const double a)
	{
		Vector2F temp;
		temp.x = (float)(x * a);
		temp.y = (float)(y * a);
		
		return temp;
	}

	Vector2F& Vector2F::operator*(const double a) const
	{
		Vector2F temp;
		temp.x = (float)(x * a);
		temp.y = (float)(y * a);
		
		return temp;
	}

	Vector2F& Vector2F::operator+(const Vector2F& v)
	{
		Vector2F temp;
		temp.x = x + v.x;
		temp.y = y + v.y;
		
		return temp;
	}

	Vector2F& Vector2F::operator+=(const Vector2F& v)
	{
		x += v.x;
		y += v.y;

		return *this;
	}

	Vector2F& Vector2F::operator+=(const float f)
	{
		x += f;
		y += f;

		return *this;
	}
}